# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Track do
  it 'reads the file' do
    read_file = ReadGpx.new(file: './spec/example_files/small.gpx')
    read_file.read

    track = described_class.new(track: read_file.tracks.first)

    expect(track.points).not_to be_empty
    expect(track.distance_in_straigh_line).to eq 28.44
    expect(track.back_to_start?).to be(false)
  end

  it 'shows maximum distance from start' do
    read_file = ReadGpx.new(file: './spec/example_files/small.gpx')
    read_file.read

    track = described_class.new(track: read_file.tracks.first)
    expect(track.max_distance_from_the_start).to eq 29.649
  end

  it 'shows point per distanse ratio' do
    read_file = ReadGpx.new(file: './spec/example_files/tiny.gpx')
    read_file.read

    track = described_class.new(track: read_file.tracks.first)
    expect(track.pt_per_km).to eq 356
  end

  it 'shows point per distanse ratio - larger route' do
    read_file = ReadGpx.new(file: './spec/example_files/small.gpx')
    read_file.read

    track = described_class.new(track: read_file.tracks.first)
    expect(track.pt_per_km).to eq 311
  end

  it 'verifies the round track' do
    read_file = ReadGpx.new(file: './spec/example_files/round.gpx')
    read_file.read

    track = described_class.new(track: read_file.tracks.first)
    expect(track.back_to_start?).to be(true)
  end

  it 'counts total ascend' do
    read_file = ReadGpx.new(file: './spec/example_files/small.gpx')
    read_file.read

    track = described_class.new(track: read_file.tracks.first)
    expect(track.total_elevation_gain).to eq 2215
  end

  context 'with lat/lng optimization' do
    it 'returns points in simple format' do
      read_file = ReadGpx.new(file: './spec/example_files/tiny.gpx')
      read_file.read

      track = described_class.new(track: read_file.tracks.first)
      points_array = track.points_in_lan_lgt_format
      expect(points_array.size).to eq 98
      expect(points_array.first).to eq [50.819746, 15.446056]
    end

    it 'counts the average points' do
      read_file = ReadGpx.new(file: './spec/example_files/tiny.gpx')
      read_file.read

      track = described_class.new(track: read_file.tracks.first)
      avg_pts = track.averaged_points
      expect(avg_pts.size).to eq 1
      expect(avg_pts.first).to eq [50.81886193877551, 15.44750430612245]
    end

    it 'averaging depending on length, not on point number' do
      read_file = ReadGpx.new(file: './spec/example_files/gsb.gpx')
      read_file.read

      track = described_class.new(track: read_file.tracks.first)
      read_file = ReadGpx.new(file: './spec/example_files/gsb2.gpx')
      read_file.read

      track2 = described_class.new(track: read_file.tracks.first)
      avg_pts = track.averaged_points
      avg_pts2 = track2.averaged_points

      expect((avg_pts2.size / avg_pts.size * 5) < (track2.points.size / track.points.size)).to be(true)
    end
  end
end
